@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: arp-ioctl.c,v 1.9 2001/12/29 21:06:41 dugsong Exp $
+ * $Id: arp-ioctl.c,v 1.15 2002/01/20 21:23:27 dugsong Exp $
  */
 
 #include "config.h"
@@ -61,42 +61,34 @@ arp_open(void)
 		return (NULL);
 
 #ifdef HAVE_ARPREQ_ARP_DEV
-	if ((a->intf = intf_open()) == NULL) {
-		free(a);
-		return (NULL);
-	}
+	if ((a->intf = intf_open()) == NULL)
+		return (arp_close(a));
 #endif
 #ifdef HAVE_STREAMS_MIB2
-	if ((a->fd = open(IP_DEV_NAME, O_RDWR)) < 0) {
-		arp_close(a);
-		return (NULL);
-	}
+	if ((a->fd = open(IP_DEV_NAME, O_RDWR)) < 0)
+		return (arp_close(a));
 #elif defined(HAVE_STREAMS_ROUTE)
-	if ((a->fd = open("/dev/route", O_WRONLY, 0)) < 0) {
-		arp_close(a);
-		return (NULL);
-	}
+	if ((a->fd = open("/dev/route", O_WRONLY, 0)) < 0)
+		return (arp_close(a));
 #else
-	if ((a->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-		arp_close(a);
-		return (NULL);
-	}
+	if ((a->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+		return (arp_close(a));
 #endif
 	return (a);
 }
 
 #ifdef HAVE_ARPREQ_ARP_DEV
 static int
-arp_set_dev(char *device, struct addr *pa, int flags, void *arg)
+arp_set_dev(const char *device, const struct intf_info *info, void *arg)
 {
 	struct arpreq *ar = (struct arpreq *)arg;
 	struct addr dst;
-	u_int32_t mask;
+	uint32_t mask;
 	
-	addr_btom(pa->addr_bits, &mask, IP_ADDR_LEN);
+	addr_btom(info->intf_addr.addr_bits, &mask, IP_ADDR_LEN);
 	addr_ston((struct sockaddr *)&ar->arp_pa, &dst);
 	
-	if ((pa->addr_ip & mask) == (dst.addr_ip & mask)) {
+	if ((info->intf_addr.addr_ip & mask) == (dst.addr_ip & mask)) {
 		strlcpy(ar->arp_dev, device, sizeof(ar->arp_dev));
 		return (1);
 	}
@@ -105,7 +97,7 @@ arp_set_dev(char *device, struct addr *pa, int flags, void *arg)
 #endif
 
 int
-arp_add(arp_t *a, struct addr *pa, struct addr *ha)
+arp_add(arp_t *a, const struct addr *pa, const struct addr *ha)
 {
 	struct arpreq ar;
 
@@ -168,7 +160,7 @@ arp_add(arp_t *a, struct addr *pa, struct addr *ha)
 }
 
 int
-arp_delete(arp_t *a, struct addr *pa)
+arp_delete(arp_t *a, const struct addr *pa)
 {
 	struct arpreq ar;
 
@@ -184,7 +176,7 @@ arp_delete(arp_t *a, struct addr *pa)
 }
 
 int
-arp_get(arp_t *a, struct addr *pa, struct addr *ha)
+arp_get(arp_t *a, const struct addr *pa, struct addr *ha)
 {
 	struct arpreq ar;
 
@@ -392,21 +384,22 @@ arp_loop(arp_t *r, arp_handler callback, void *arg)
 int
 arp_loop(arp_t *a, arp_handler callback, void *arg)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 #endif
 
-int
+arp_t *
 arp_close(arp_t *a)
 {
 	assert(a != NULL);
-
-	if (a->fd != 0 && close(a->fd) < 0)
-		return (-1);
+	
+	if (a->fd > 0)
+		close(a->fd);
 #ifdef HAVE_ARPREQ_ARP_DEV
-	if (intf_close(a->intf) < 0)
-		return (-1);
+	if (a->intf != NULL)
+		intf_close(a->intf);
 #endif
-	return (0);
+	free(a);
+	return (NULL);
 }
