@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: fw-ipfw.c,v 1.9 2001/12/29 21:06:41 dugsong Exp $
+ * $Id: fw-ipfw.c,v 1.13 2002/01/20 21:23:28 dugsong Exp $
  */
 
 #include "config.h"
@@ -39,67 +39,67 @@ fr_to_ipfw_device(char *device, char *name, short *unit)
 }
 
 static void
-fr_to_ipfw(struct fw_rule *fr, struct ip_fw *ipfw)
+fr_to_ipfw(const struct fw_rule *fr, struct ip_fw *ipfw)
 {
 	int i;
 	
 	memset(ipfw, 0, sizeof(*ipfw));
 
-	if (fr->direction == FW_DIR_IN) {
-		if (*fr->device != '\0') {
-			fr_to_ipfw_device(fr->device,
+	if (fr->fw_dir == FW_DIR_IN) {
+		if (*fr->fw_device != '\0') {
+			fr_to_ipfw_device(fr->fw_device,
 			    ipfw->fw_in_if.fu_via_if.name,
 			    &ipfw->fw_in_if.fu_via_if.unit);
 			ipfw->fw_flg |= IP_FW_F_IIFNAME;
 		}
 		ipfw->fw_flg |= IP_FW_F_IN;
 	} else {
-		if (*fr->device != '\0') {
-			fr_to_ipfw_device(fr->device,
+		if (*fr->fw_device != '\0') {
+			fr_to_ipfw_device(fr->fw_device,
 			    ipfw->fw_out_if.fu_via_if.name,
 			    &ipfw->fw_out_if.fu_via_if.unit);
 			ipfw->fw_flg |= IP_FW_F_OIFNAME;
 		}
 		ipfw->fw_flg |= IP_FW_F_OUT;
 	}
-	if (fr->op == FW_OP_ALLOW)
+	if (fr->fw_op == FW_OP_ALLOW)
 		ipfw->fw_flg |= IP_FW_F_ACCEPT;
 	else
 		ipfw->fw_flg |= IP_FW_F_DENY;
 	
-	ipfw->fw_prot = fr->proto;
-	ipfw->fw_src.s_addr = fr->src.addr_ip;
-	ipfw->fw_dst.s_addr = fr->dst.addr_ip;
-	addr_btom(fr->src.addr_bits, &ipfw->fw_smsk.s_addr, IP_ADDR_LEN);
-	addr_btom(fr->dst.addr_bits, &ipfw->fw_dmsk.s_addr, IP_ADDR_LEN);
+	ipfw->fw_prot = fr->fw_proto;
+	ipfw->fw_src.s_addr = fr->fw_src.addr_ip;
+	ipfw->fw_dst.s_addr = fr->fw_dst.addr_ip;
+	addr_btom(fr->fw_src.addr_bits, &ipfw->fw_smsk.s_addr, IP_ADDR_LEN);
+	addr_btom(fr->fw_dst.addr_bits, &ipfw->fw_dmsk.s_addr, IP_ADDR_LEN);
 
-	switch (fr->proto) {
+	switch (fr->fw_proto) {
 	case IP_PROTO_TCP:
 	case IP_PROTO_UDP:
 		i = 0;
-		if (fr->sport[0] != fr->sport[1]) {
+		if (fr->fw_sport[0] != fr->fw_sport[1]) {
 			ipfw->fw_flg |= IP_FW_F_SRNG;
-			ipfw->fw_uar.fw_pts[i++] = fr->sport[0];
-			ipfw->fw_uar.fw_pts[i++] = fr->sport[1];
+			ipfw->fw_uar.fw_pts[i++] = fr->fw_sport[0];
+			ipfw->fw_uar.fw_pts[i++] = fr->fw_sport[1];
 			IP_FW_SETNSRCP(ipfw, 2);
-		} else if (fr->sport[0] > 0) {
-			ipfw->fw_uar.fw_pts[i++] = fr->sport[0];
+		} else if (fr->fw_sport[0] > 0) {
+			ipfw->fw_uar.fw_pts[i++] = fr->fw_sport[0];
 			IP_FW_SETNSRCP(ipfw, 1);
 		}
-		if (fr->dport[0] != fr->dport[1]) {
+		if (fr->fw_dport[0] != fr->fw_dport[1]) {
 			ipfw->fw_flg |= IP_FW_F_DRNG;
-			ipfw->fw_uar.fw_pts[i++] = fr->dport[0];
-			ipfw->fw_uar.fw_pts[i++] = fr->dport[1];
+			ipfw->fw_uar.fw_pts[i++] = fr->fw_dport[0];
+			ipfw->fw_uar.fw_pts[i++] = fr->fw_dport[1];
 			IP_FW_SETNDSTP(ipfw, 2);
-		} else if (fr->dport[0] > 0) {
-			ipfw->fw_uar.fw_pts[i++] = fr->dport[0];
+		} else if (fr->fw_dport[0] > 0) {
+			ipfw->fw_uar.fw_pts[i++] = fr->fw_dport[0];
 			IP_FW_SETNDSTP(ipfw, 1);
 		}
 		break;
 	case IP_PROTO_ICMP:
-		if (fr->sport[1]) {
-			ipfw->fw_uar.fw_icmptypes[fr->sport[0] / 32] |=
-			    1 << (fr->sport[0] % 32);
+		if (fr->fw_sport[1]) {
+			ipfw->fw_uar.fw_icmptypes[fr->fw_sport[0] / 32] |=
+			    1 << (fr->fw_sport[0] % 32);
 			ipfw->fw_flg |= IP_FW_F_ICMPBIT;
 		}
 		/* XXX - no support for ICMP code. */
@@ -108,57 +108,60 @@ fr_to_ipfw(struct fw_rule *fr, struct ip_fw *ipfw)
 }
 
 static void
-ipfw_to_fr(struct ip_fw *ipfw, struct fw_rule *fr)
+ipfw_to_fr(const struct ip_fw *ipfw, struct fw_rule *fr)
 {
 	int i;
 	
 	memset(fr, 0, sizeof(*fr));
 
 	if ((ipfw->fw_flg & IP_FW_F_IN) && *ipfw->fw_in_if.fu_via_if.name)
-		snprintf(fr->device, sizeof(fr->device), "%s%d",
+		snprintf(fr->fw_device, sizeof(fr->fw_device), "%s%d",
 		    ipfw->fw_in_if.fu_via_if.name,
 		    ipfw->fw_in_if.fu_via_if.unit);
 	else if ((ipfw->fw_flg & IP_FW_F_OUT) &&
 	    *ipfw->fw_out_if.fu_via_if.name)
-		snprintf(fr->device, sizeof(fr->device), "%s%d",
+		snprintf(fr->fw_device, sizeof(fr->fw_device), "%s%d",
 		    ipfw->fw_out_if.fu_via_if.name,
 		    ipfw->fw_out_if.fu_via_if.unit);
 	
-	fr->op = (ipfw->fw_flg & IP_FW_F_ACCEPT) ? FW_OP_ALLOW : FW_OP_BLOCK;
-	fr->direction = (ipfw->fw_flg & IP_FW_F_IN) ? FW_DIR_IN : FW_DIR_OUT;
-	fr->proto = ipfw->fw_prot;
+	fr->fw_op = (ipfw->fw_flg & IP_FW_F_ACCEPT) ?
+	    FW_OP_ALLOW : FW_OP_BLOCK;
+	fr->fw_dir = (ipfw->fw_flg & IP_FW_F_IN) ? FW_DIR_IN : FW_DIR_OUT;
+	fr->fw_proto = ipfw->fw_prot;
 
-	fr->src.addr_type = fr->dst.addr_type = ADDR_TYPE_IP;
-	fr->src.addr_ip = ipfw->fw_src.s_addr;
-	fr->dst.addr_ip = ipfw->fw_dst.s_addr;
-	addr_mtob(&ipfw->fw_smsk.s_addr, IP_ADDR_LEN, &fr->src.addr_bits);
-	addr_mtob(&ipfw->fw_dmsk.s_addr, IP_ADDR_LEN, &fr->dst.addr_bits);
+	fr->fw_src.addr_type = fr->fw_dst.addr_type = ADDR_TYPE_IP;
+	fr->fw_src.addr_ip = ipfw->fw_src.s_addr;
+	fr->fw_dst.addr_ip = ipfw->fw_dst.s_addr;
+	addr_mtob(&ipfw->fw_smsk.s_addr, IP_ADDR_LEN, &fr->fw_src.addr_bits);
+	addr_mtob(&ipfw->fw_dmsk.s_addr, IP_ADDR_LEN, &fr->fw_dst.addr_bits);
 
-	switch (fr->proto) {
+	switch (fr->fw_proto) {
 	case IP_PROTO_TCP:
 	case IP_PROTO_UDP:
 		if ((ipfw->fw_flg & IP_FW_F_SRNG) &&
 		    IP_FW_GETNSRCP(ipfw) == 2) {
-			fr->sport[0] = ipfw->fw_uar.fw_pts[0];
-			fr->sport[1] = ipfw->fw_uar.fw_pts[1];
+			fr->fw_sport[0] = ipfw->fw_uar.fw_pts[0];
+			fr->fw_sport[1] = ipfw->fw_uar.fw_pts[1];
 		} else if (IP_FW_GETNSRCP(ipfw) == 1) {
-			fr->sport[0] = fr->sport[1] = ipfw->fw_uar.fw_pts[0];
+			fr->fw_sport[0] = fr->fw_sport[1] =
+			    ipfw->fw_uar.fw_pts[0];
 		} else if (IP_FW_GETNSRCP(ipfw) == 0) {
-		  	fr->sport[0] = 0;
-			fr->sport[1] = TCP_PORT_MAX;
+		  	fr->fw_sport[0] = 0;
+			fr->fw_sport[1] = TCP_PORT_MAX;
 		}
 		
 		if ((ipfw->fw_flg & IP_FW_F_DRNG) &&
 		    IP_FW_GETNDSTP(ipfw) == 2) {
 			i = IP_FW_GETNSRCP(ipfw);
-			fr->dport[0] = ipfw->fw_uar.fw_pts[i];
-			fr->dport[1] = ipfw->fw_uar.fw_pts[i + 1];
+			fr->fw_dport[0] = ipfw->fw_uar.fw_pts[i];
+			fr->fw_dport[1] = ipfw->fw_uar.fw_pts[i + 1];
 		} else if (IP_FW_GETNDSTP(ipfw) == 1) {
 			i = IP_FW_GETNSRCP(ipfw);
-			fr->dport[0] = fr->dport[1] = ipfw->fw_uar.fw_pts[i];
+			fr->fw_dport[0] = fr->fw_dport[1] =
+			    ipfw->fw_uar.fw_pts[i];
 		} else if (IP_FW_GETNDSTP(ipfw) == 0) {
-		  	fr->dport[0] = 0;
-			fr->dport[1] = TCP_PORT_MAX;
+		  	fr->fw_dport[0] = 0;
+			fr->fw_dport[1] = TCP_PORT_MAX;
 		}
 		break;
 	case IP_PROTO_ICMP:
@@ -166,8 +169,8 @@ ipfw_to_fr(struct ip_fw *ipfw, struct fw_rule *fr)
 			for (i = 0; i < IP_FW_ICMPTYPES_DIM * 32; i++) {
 				if (ipfw->fw_uar.fw_icmptypes[i / 32] &
 				    (1U << (i % 32))) {
-					fr->sport[0] = i;
-					fr->sport[1] = 0xff;
+					fr->fw_sport[0] = i;
+					fr->fw_sport[1] = 0xff;
 					break;
 				}
 			}
@@ -185,15 +188,14 @@ fw_open(void)
 	if ((fw = calloc(1, sizeof(*fw))) == NULL)
 		return (NULL);
 
-	if ((fw->fd = socket(AF_INET, SOCK_RAW, IPPROTO_IP)) < 0) {
-		free(fw);
-		return (NULL);
-	}
+	if ((fw->fd = socket(AF_INET, SOCK_RAW, IPPROTO_IP)) < 0)
+		return (fw_close(fw));
+	
 	return (fw);
 }
 
 int
-fw_add(fw_t *fw, struct fw_rule *rule)
+fw_add(fw_t *fw, const struct fw_rule *rule)
 {
 	struct ip_fw ipfw;
 	
@@ -206,20 +208,20 @@ fw_add(fw_t *fw, struct fw_rule *rule)
 }
 
 static int
-fw_cmp(struct fw_rule *a, struct fw_rule *b)
+fw_cmp(const struct fw_rule *a, const struct fw_rule *b)
 {
-	if (strcmp(a->device, b->device) != 0 || a->op != b->op ||
-	    a->direction != b->direction || a->proto != b->proto || 
-	    addr_cmp(&a->src, &b->src) != 0 ||
-	    addr_cmp(&a->dst, &b->dst) != 0 ||
-	    memcmp(a->sport, b->sport, sizeof(a->sport)) != 0 ||
-	    memcmp(a->dport, b->dport, sizeof(a->dport)) != 0)
+	if (strcmp(a->fw_device, b->fw_device) != 0 || a->fw_op != b->fw_op ||
+	    a->fw_dir != b->fw_dir || a->fw_proto != b->fw_proto || 
+	    addr_cmp(&a->fw_src, &b->fw_src) != 0 ||
+	    addr_cmp(&a->fw_dst, &b->fw_dst) != 0 ||
+	    memcmp(a->fw_sport, b->fw_sport, sizeof(a->fw_sport)) != 0 ||
+	    memcmp(a->fw_dport, b->fw_dport, sizeof(a->fw_dport)) != 0)
 		return (-1);
 	return (0);
 }
 
 int
-fw_delete(fw_t *fw, struct fw_rule *rule)
+fw_delete(fw_t *fw, const struct fw_rule *rule)
 {
 	struct ip_fw *ipfw;
 	struct fw_rule fr;
@@ -247,24 +249,28 @@ fw_delete(fw_t *fw, struct fw_rule *rule)
 			return (-1);
 		}
 	}
-	ret = 0;
+	ret = -1;
 
 	/* XXX - 65535 is the fixed ipfw default rule. */
 	for (ipfw = (struct ip_fw *)buf; ipfw->fw_number < 65535; ipfw++) {
 		ipfw_to_fr(ipfw, &fr);
 		if (fw_cmp(&fr, rule) == 0) {
 			if (setsockopt(fw->fd, IPPROTO_IP, IP_FW_DEL,
-			    ipfw, sizeof(*ipfw)) < 0) {
-				free(buf);
-				return (-1);
-			}
-			free(buf);
-			return (0);
+			    ipfw, sizeof(*ipfw)) < 0)
+				ret = -2;
+			else
+				ret = 0;
+			break;
 		}
 	}
-	errno = ESRCH;
 	free(buf);
-	return (-1);
+	
+	if (ret < 0) {
+		if (ret == -1)
+			errno = ESRCH;
+		return (-1);
+	}
+	return (0);
 }
 
 int
@@ -307,14 +313,13 @@ fw_loop(fw_t *fw, fw_handler callback, void *arg)
 	return (ret);
 }
 
-int
+fw_t *
 fw_close(fw_t *fw)
 {
 	assert(fw != NULL);
 
-	if (close(fw->fd) < 0)
-		return (-1);
-
+	if (fw->fd > 0)
+		close(fw->fd);
 	free(fw);
-	return (0);
+	return (NULL);
 }
