@@ -3,18 +3,13 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: ip.c,v 1.11 2001/12/18 08:30:56 dugsong Exp $
+ * $Id: ip.c,v 1.24 2002/01/20 21:23:28 dugsong Exp $
  */
 
 #include "config.h"
 
-#include <sys/param.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-
 #include <netinet/in.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,11 +51,9 @@ ip_open(void)
 	
 	if ((i->intf = intf_open()) == NULL ||
 	    (i->arp = arp_open()) == NULL ||
-	    (i->route = route_open()) == NULL) {
-		ip_close(i);
-		free(i);
-		return (NULL);
-	}
+	    (i->route = route_open()) == NULL)
+		return (ip_close(i));
+	
 	return (i);
 }
 #else /* !HAVE_RAWIP_COOKED */
@@ -68,62 +61,57 @@ ip_t *
 ip_open(void)
 {
 	ip_t *i;
-	int n, fd, len;
+	int n, len;
 
-	if ((fd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0)
+	if ((i = calloc(1, sizeof(*i))) == NULL)
 		return (NULL);
 
+	if ((i->fd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0)
+		return (ip_close(i));
+#ifdef IP_HDRINCL
 	n = 1;
-	if (setsockopt(fd, IPPROTO_IP, IP_HDRINCL, &n, sizeof(n)) < 0) {
-		close(fd);
-		return (NULL);
-	}
+	if (setsockopt(i->fd, IPPROTO_IP, IP_HDRINCL, &n, sizeof(n)) < 0)
+		return (ip_close(i));
+#endif
 #ifdef SO_SNDBUF
 	len = sizeof(n);
-	if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &n, &len) < 0) {
-		close(fd);
-		return (NULL);
-	}
+	if (getsockopt(i->fd, SOL_SOCKET, SO_SNDBUF, &n, &len) < 0)
+		return (ip_close(i));
+
 	for (n += 128; n < 1048576; n += 128) {
-		if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &n, len) < 0) {
+		if (setsockopt(i->fd, SOL_SOCKET, SO_SNDBUF, &n, len) < 0) {
 			if (errno == ENOBUFS)
 				break;
-			close(fd);
-			return (NULL);
+			return (ip_close(i));
 		}
 	}
 #endif
 #ifdef SO_BROADCAST
 	n = 1;
-	if (setsockopt(fd, SOL_SOCKET, SO_BROADCAST, &n, sizeof(n)) < 0) {
-		close(fd);
-		return (NULL);
-	}
+	if (setsockopt(i->fd, SOL_SOCKET, SO_BROADCAST, &n, sizeof(n)) < 0)
+		return (ip_close(i));
 #endif
-	if ((i = malloc(sizeof(*i))) == NULL) {
-		close(fd);
-		return (NULL);
-	}
-	i->fd = fd;
 	return (i);
 }
 #endif /* !HAVE_RAWIP_COOKED */
 
 #ifdef HAVE_RAWIP_COOKED
 static int
-ip_match_intf(char *device, struct addr *addr, int flags, void *arg)
+ip_match_intf(const char *device, const struct intf_info *info, void *arg)
 {
 	ip_t *i = (ip_t *)arg;
 	
-	if (addr->addr_ip == i->ip_src.addr_ip ||
+	if (info->intf_addr.addr_ip == i->ip_src.addr_ip ||
 	    i->ip_src.addr_ip == IP_ADDR_ANY) {
-		if (intf_get(i->intf, device, &i->eth_src, NULL) == 0) {
-			if (i->eth != NULL)
-				eth_close(i->eth);
-			if ((i->eth = eth_open(device)) == NULL)
-				return (-1);
-			return (1);
+		if (i->eth != NULL)
+			i->eth = eth_close(i->eth);
+		if ((i->eth = eth_open(device)) == NULL)
+			return (-1);
+		if (eth_get(i->eth, &i->eth_src.addr_eth) < 0) {
+			i->eth = eth_close(i->eth);
+			return (-1);
 		}
+		return (1);
 	}
 	return (0);
 }
@@ -170,7 +158,7 @@ ip_lookup(ip_t *i, ip_addr_t dst)
 	return (0);
 }
 
-ssize_t
+size_t
 ip_send(ip_t *i, const void *buf, size_t len)
 {
 	struct ip_hdr *ip;
@@ -229,7 +217,7 @@ ip_send(ip_t *i, const void *buf, size_t len)
 	return (len);
 }
 #else /* !HAVE_RAWIP_COOKED */
-ssize_t
+size_t
 ip_send(ip_t *i, const void *buf, size_t len)
 {
 	struct ip_hdr *ip;
@@ -262,159 +250,19 @@ ip_send(ip_t *i, const void *buf, size_t len)
 }
 #endif /* !HAVE_RAWIP_COOKED */
 
-int
+ip_t *
 ip_close(ip_t *i)
 {
-	assert(i != NULL);
-
-	if (close(i->fd) < 0)
-		return (-1);
+	if (i->fd > 0)
+		close(i->fd);
 #ifdef HAVE_RAWIP_COOKED	
 	if (i->intf != NULL)
 		intf_close(i->intf);
-
 	if (i->arp != NULL)
 		arp_close(i->arp);
-
 	if (i->route != NULL)
 		route_close(i->route);
 #endif
 	free(i);
-	return (0);
-}
-
-size_t
-ip_add_opt(void *buf, size_t len, const void *optbuf, size_t optlen)
-{
-	struct ip_hdr *ip;
-	struct ip_opt *opt;
-	u_char *p;
-	int datalen, padlen;
-
-	ip = (struct ip_hdr *)buf;
-	p = (u_char *)ip + (ip->ip_hl << 2);
-	datalen = ntohs(ip->ip_len) - (p - (u_char *)buf);
-	
-	if ((padlen = 4 - (optlen % 4)) == 4)
-		padlen = 0;
-	
-	assert(ntohs(ip->ip_len) + optlen + padlen < len);
-	assert((ip->ip_hl << 2) + optlen + padlen < IP_HDR_LEN_MAX);
-	
-	opt = (struct ip_opt *)optbuf;
-	if (IP_OPT_TYPEONLY(opt->opt_type))
-		optlen = 1;
-	else
-		assert(opt->opt_len == optlen);
-
-	/* XXX - shift any existing IP data. */
-	if (datalen) {
-		memmove(p + optlen + padlen, p, datalen);
-	}
-	memmove(p, optbuf, optlen);
-	p += optlen;
-	
-	/* XXX - pad with NOPs to word boundary. */
-	if (padlen) {
-		memset(p, IP_OPT_NOP, padlen);
-		p += padlen;
-		optlen += padlen;
-	}
-	ip->ip_hl = (p - (u_char *)ip) >> 2;
-	ip->ip_len = htons(ntohs(ip->ip_len) + optlen);
-	
-	return (optlen);
-}
-
-void
-ip_checksum(void *buf, size_t len)
-{
-	struct ip_hdr *ip;
-	int hl, sum;
-
-	ip = (struct ip_hdr *)buf;
-	hl = ip->ip_hl << 2;
-	
-	assert(hl <= IP_HDR_LEN_MAX);
-	assert(ntohs(ip->ip_len) <= len);
-	
-	len = ntohs(ip->ip_len) - hl;
-	
-	ip->ip_sum = 0;
-	sum = ip_cksum_add(ip, hl, 0);
-	ip->ip_sum = ip_cksum_carry(sum);
-	
-	if (ip->ip_p == IP_PROTO_TCP && len >= TCP_HDR_LEN) {
-		struct tcp_hdr *tcp = (struct tcp_hdr *)((u_char *)ip + hl);
-		
-		tcp->th_sum = 0;
-		sum = ip_cksum_add(tcp, len, 0) + htons(ip->ip_p + len);
-		sum = ip_cksum_add(&ip->ip_src, 8, sum);
-		tcp->th_sum = ip_cksum_carry(sum);
-	} else if (ip->ip_p == IP_PROTO_UDP && len >= UDP_HDR_LEN) {
-		struct udp_hdr *udp = (struct udp_hdr *)((u_char *)ip + hl);
-
-		udp->uh_sum = 0;
-		sum = ip_cksum_add(udp, len, 0) + htons(ip->ip_p + len);
-		sum = ip_cksum_add(&ip->ip_src, 8, sum);
-		udp->uh_sum = ip_cksum_carry(sum);
-	} else if ((ip->ip_p == IP_PROTO_ICMP || ip->ip_p == IP_PROTO_IGMP) &&
-	    len >= ICMP_HDR_LEN) {
-		struct icmp_hdr *icmp = (struct icmp_hdr *)((u_char *)ip + hl);
-		
-		icmp->icmp_cksum = 0;
-		sum = ip_cksum_add(icmp, len, 0);
-		icmp->icmp_cksum = ip_cksum_carry(sum);
-	}
-}
-
-int
-ip_cksum_add(void *buf, size_t len, int cksum)
-{
-	u_short *sp = (u_short *)buf;
-	int n, sn;
-	
-	sn = len / 2;
-	n = (sn + 15) / 16;
-
-	/* XXX - unroll loop using Duff's device. */
-	switch (sn % 16) {
-	case 0:	do {
-		cksum += *sp++;
-	case 15:
-		cksum += *sp++;
-	case 14:
-		cksum += *sp++;
-	case 13:
-		cksum += *sp++;
-	case 12:
-		cksum += *sp++;
-	case 11:
-		cksum += *sp++;
-	case 10:
-		cksum += *sp++;
-	case 9:
-		cksum += *sp++;
-	case 8:
-		cksum += *sp++;
-	case 7:
-		cksum += *sp++;
-	case 6:
-		cksum += *sp++;
-	case 5:
-		cksum += *sp++;
-	case 4:
-		cksum += *sp++;
-	case 3:
-		cksum += *sp++;
-	case 2:
-		cksum += *sp++;
-	case 1:
-		cksum += *sp++;
-		} while (--n > 0);
-	}
-	if (len & 1)
-		cksum += htons(*(u_char *)sp << 8);
-
-	return (cksum);
+	return (NULL);
 }
