@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: route-none.c,v 1.1.1.1 2001/10/11 04:14:55 dugsong Exp $
+ * $Id: route-none.c,v 1.4 2002/01/20 21:23:28 dugsong Exp $
  */
 
 #include "config.h"
@@ -19,41 +19,40 @@
 route_t *
 route_open(void)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (NULL);
 }
 
 int
-route_add(route_t *r, struct addr *dst, struct addr *gw)
+route_add(route_t *r, const struct addr *dst, const struct addr *gw)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
 int
-route_delete(route_t *r, struct addr *dst)
+route_delete(route_t *r, const struct addr *dst)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
 int
-route_get(route_t *r, struct addr *dst, struct addr *gw)
+route_get(route_t *r, const struct addr *dst, struct addr *gw)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
 int
 route_loop(route_t *r, route_handler callback, void *arg)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
-int
+route_t *
 route_close(route_t *r)
 {
-	errno = EOPNOTSUPP;
-	return (-1);
+	return (NULL);
 }
